#include "Bomb.h"


Bomb::Bomb(const sf::Texture *texture, TileMap *tab)
	: sf::Sprite(*texture), 
	m_Texture(texture), 
	m_map(tab), 
	m_isAlive(true), 
	m_isExploding(false), 
	m_currentFrame(0),
	m_speed(SPEED_FIRE)
{
	m_nbFrames = this->GetTexture()->GetWidth() / BOMB_W;
}

Bomb::~Bomb(void)
{
}

void Bomb::Update(float timeFactor, Tank& enemyTank)
{
	// Raccourcis pour les valeurs
	float Bx = this->GetPosition().x;
	float By = this->GetPosition().y;
	float Bw = BOMB_W;
	float Bh = BOMB_MH;
	float Br = this->GetRotation();

	OBB tile, bomb(Bx, By + 3, Bw, Bh - 6, Br);

	// Dplacement
	this->Move(static_cast<float>(cos(PI * Br / 180) * m_speed) * timeFactor, 
		static_cast<float>(sin(PI * Br / 180) * m_speed) * timeFactor);

	// Supprime le missile si il est hors-cran
	if (Bx < 0 || Bx >= SCREEN_WIDTH ||	By < 0 || By >= SCREEN_HEIGHT)
		this->m_isAlive = false;

	else // Sinon test de colision
	{
		// Test de colison avec l'autre tank
		if (Collision(&bomb, &enemyTank.GetOBB()))
		{
			enemyTank.TakeDamages(10); // fois nb de boucles
			m_speed = 0.f;
			this->m_isExploding = true;
		}

		// Test de colision avec les tiles
		for (unsigned int i = static_cast<unsigned int>(((Bx / CELL_SIZE) - ((Bw / CELL_SIZE))) - 1); 
			i < static_cast<unsigned int>(((Bx / CELL_SIZE) + ((Bw / CELL_SIZE))) + 1); 
			++i)
		{
			if (i < 0) continue;
			if (i >= m_map->GetMap().size()) break;
			for (unsigned int j = static_cast<unsigned int>(((By / CELL_SIZE) - ((Bh / CELL_SIZE))) - 1); 
				j < static_cast<unsigned int>(((By / CELL_SIZE) + ((Bh / CELL_SIZE))) + 1); 
				++j)
			{
				if (j < 0) continue;
				if (j >= m_map->GetNbColumns()) break;
				else if (!m_map->GetMap().at(i).at(j).IsHoverable()) // Le tile n'est pas survolable
				{
					// Cre l'OBB de la tile  tester actuelle et de la bombe actuelle
					tile = OBB(i * CELL_SIZE + CELL_SIZE / 2, j * CELL_SIZE + CELL_SIZE / 2, CELL_SIZE, CELL_SIZE);

					// Si il y a colision, on explose la bombe
					if (Collision(&bomb, &tile))
					{
  						m_map->AddToTileLife(i, j, -10); // fois nb de boucle
						m_speed = 0.f;
						this->m_isExploding = true;
					}
				}
			}
		}
	}
}