#include "OBB.h"
#include <cmath>

OBB::OBB(double oX, double oY, double w, double h, double angle)
	: AABB(oX,oY,w,h), angle(angle)
{
}

OBB::~OBB(void)
{
}

bool Collision(OBB* pRect1, OBB* pRect2)
{
	OBB rectA(*pRect1);
	OBB rectB(*pRect2);
	rectB.RotateAround_private(&rectA);

	if(!Collision(&rectA, &rectB.outLineRect))
		return false;

	rectB = *pRect2;
	rectA.RotateAround_private(&rectB);

	if(!Collision(&rectA.outLineRect, &rectB))
		return false;

	return true;
}

void OBB::SetAngle(double new_angle)
{
	angle = new_angle;
}

void OBB::UpdateOutLineRect_private()
{
	double absCosA = abs(cos( angle * DEG_TO_RAD));
	double absSinA = abs(sin( angle * DEG_TO_RAD));
	outLineRect.w = w * absCosA + h * absSinA;
	outLineRect.h = w * absSinA + h * absCosA;
	outLineRect.oX = oX;
	outLineRect.oY = oY;
}

void OBB::RotateAround_private(const OBB* base)
{
	double cosA = cos( -base->angle * DEG_TO_RAD);
	double sinA = sin( -base->angle * DEG_TO_RAD);
	double oXDif = oX - base->oX;
	double oYDif = oY - base->oY;
	double oXDif2 = oXDif * cosA - oYDif * sinA;
	double oYDif2 = oXDif * sinA + oYDif * cosA;
	oX = base->oX + oXDif2;
	oY = base->oY + oYDif2;
	angle -= base->angle;
	UpdateOutLineRect_private();
}