#include "Tank.h"


Tank::Tank(const sf::Texture& textureTank, const sf::Texture& textureBomb, const sf::Vector2f& pos, TileMap &map)
	: m_textureTank(textureTank),	// Copie l'adresse de la texture du tank
	m_textureBomb(textureBomb),		// Copie l'adresse de la texture des bombes
	m_map(map),						// Copie l'adresse du tableau de jeu
	m_moveSpeed(SPEED),				// Dfinie la vitesse de dplacement du tank
	m_rotateSpeed(SPEED_R),			// Dfinie la vitesse de ratation du tank
	m_currentFrameX(0),				// Dfinie la frame en X actuelle de l'animation
	m_currentFrameY(0),				// Dfinie la frame en Y actuelle de l'animation
	m_isMovingForw(false),			// Si le tank est en train d'avancer
	m_isMovingBckw(false),			// Si le tank est en train de reculer
	m_isExploding(false),			// Si le tank est en train d'exploser
	m_life(2000)					// Vie du Tank
{
	// Change la texture
	this->SetTexture(m_textureTank);

	// Origine de l'image
	this->SetOrigin(TANK_W / 2, TANK_H / 2);

	// Nb de frames dans l'animation
	m_nbFrames = m_textureTank.GetWidth() / TANK_W;

	// Change la position initiale
	Tank::SetPosition(pos);

	// Mets  jour l'animation du tank
	UpdateAnim();
}

Tank::~Tank(void)
{
}

// Dplacement intuitif
void Tank::MoveForw(float timeFactor, Tank& enemyTank)
{
	// Animation du tank
	m_isMovingForw = true;
	UpdateAnim();

	// Bouge le tank vers l'avant en fonction de sa rotation
	Move(static_cast<float>(cos(PI * this->GetRotation() / 180) * m_moveSpeed) * timeFactor,
		static_cast<float>(sin(PI * this->GetRotation() / 180) * m_moveSpeed) * timeFactor
		, enemyTank);
}

void Tank::MoveBckw(float timeFactor, Tank& enemyTank)
{
	// Animation du tank
	m_isMovingBckw = true;
	UpdateAnim();

	// Bouge le tank vers l'arrire en fonction de sa rotation
	Move(- static_cast<float>(cos(PI * this->GetRotation() / 180) * m_moveSpeed) * timeFactor * 1.3f,
		- static_cast<float>(sin(PI * this->GetRotation() / 180) * m_moveSpeed) * timeFactor * 1.3f
		, enemyTank);
}

void Tank::TurnRight(float timeFactor, Tank& enemyTank)
{
	Rotate(+ m_rotateSpeed * timeFactor, enemyTank);
}

void Tank::TurnLeft(float timeFactor, Tank& enemyTank)
{
	Rotate(- m_rotateSpeed * timeFactor, enemyTank);
}

// Dplacement prcis
void Tank::Move(float x, float y, Tank& enemyTank)
{
	// Si on est mort, on bouge pas
	if (m_isExploding) return;
	
	// Variables de test pour X et Y
	float tmpX = x, tmpY = y;

	// Test si le dplacement est succeptible de sauter les murs (rcursivit)
	if (abs(tmpX) > CELL_SIZE)
	{
		Move(tmpX - CELL_SIZE, tmpY, enemyTank);
		tmpX = CELL_SIZE - tmpX;
	}
	if (abs(tmpY) > CELL_SIZE)
	{
		Move(tmpX, tmpY - CELL_SIZE, enemyTank);
		tmpY = CELL_SIZE - tmpY;
	}

	// Vrifie le hors-cran
	if (this->GetGlobalBounds().Left + tmpX < 0)
		tmpX -= this->GetGlobalBounds().Left;
	else if (this->GetGlobalBounds().Left + tmpX + this->GetGlobalBounds().Width > SCREEN_WIDTH)
		tmpX = SCREEN_WIDTH - this->GetGlobalBounds().Left - this->GetGlobalBounds().Width;
	if (this->GetGlobalBounds().Top + tmpY < 0)
		tmpY -= this->GetGlobalBounds().Top;
	else if (this->GetGlobalBounds().Top + tmpY + this->GetGlobalBounds().Height > SCREEN_HEIGHT)
		tmpY = SCREEN_HEIGHT - this->GetGlobalBounds().Top - this->GetGlobalBounds().Height;

	// Racourci l'appel aux fonctions
	float Tx = this->GetPosition().x;
	float Ty = this->GetPosition().y;
	float Tw = TANK_W;
	float Th = TANK_H;
	float Tr = this->GetRotation();

	// Variables de test
	OBB tmpTank;
	OBB tile;

	/* Colision avec l'autre tank */

	// Test pour le dplacement en X
	tmpTank = OBB(Tx + tmpX, Ty, Tw, Th, Tr); // Calcul de la nouvelle OBB
	while (Collision(&tmpTank, &enemyTank.GetOBB()) && tmpX != 0)
	{
		// Diminue le dplacement en X
		if (tmpX > 0.f)
			tmpX -= 0.02f;
		else
			tmpX += 0.02f;

		// Si le dpacement est trop faible, on arrete
		if (abs(tmpX) <= 0.03f)
			tmpX = 0.f;

		// Calcul de la nouvelle OBB
		tmpTank = OBB(Tx + tmpX, Ty, Tw, Th, Tr);
	}

	// Test pour le dplacement en X
	tmpTank = OBB(Tx, Ty + tmpY, Tw, Th, Tr); // Calcul de la nouvelle OBB
	while (Collision(&tmpTank, &enemyTank.GetOBB()) && tmpY != 0)
	{
		// Diminue le dplacement en X
		if (tmpY > 0.f)
			tmpY -= 0.02f;
		else
			tmpY += 0.02f;

		// Si le dpacement est trop faible, on arrete
		if (abs(tmpY) <= 0.03f)
			tmpY = 0.f;

		// Calcul de la nouvelle OBB
		tmpTank = OBB(Tx, Ty + tmpY, Tw, Th, Tr);
	}

	/* Colision avec le dcor */
	for (unsigned int i = static_cast<unsigned int>(((Tx / CELL_SIZE) - ((Tw / CELL_SIZE))) - 1); 
						i < static_cast<unsigned int>(((Tx / CELL_SIZE) + ((Tw / CELL_SIZE))) + 1); 
						++i)
	{
		if (i < 0) continue;
		if (i >= m_map.GetMap().size()) break;
		for (unsigned int j = static_cast<unsigned int>(((Ty / CELL_SIZE) - ((Th / CELL_SIZE))) - 1); 
							j < static_cast<unsigned int>(((Ty / CELL_SIZE) + ((Th / CELL_SIZE))) + 1); 
							++j)
		{
			if (j < 0) continue;
			if (j >= m_map.GetMap().at(i).size()) break;
			else if (m_map.GetMap().at(i).at(j).IsBlocking()) // Le tile est bloquant
			{
				// Cre l'OBB de la tile  tester actuelle
				// TODO: Utiliser l'AABB de la Tile
				tile = OBB(i * CELL_SIZE + CELL_SIZE / 2, j * CELL_SIZE + CELL_SIZE / 2, CELL_SIZE, CELL_SIZE);

				// Test pour le dplacement en X
				tmpTank = OBB(Tx + tmpX, Ty, Tw, Th, Tr); // Calcul de la nouvelle OBB
				while (Collision(&tmpTank, &tile) && tmpX != 0.f)
				{
					// Diminue le dplacement en X
					if (tmpX > 0.f)
						tmpX -= 0.02f;
					else
						tmpX += 0.02f;

					// Si le dpacement est trop faible, on arrete
					if (abs(tmpX) <= 0.03f)
						tmpX = 0.f;

					// Calcul de la nouvelle OBB
					tmpTank = OBB(Tx + tmpX, Ty, Tw, Th, Tr);
				}

				// Test pour le dplacement en X
				tmpTank = OBB(Tx, Ty + tmpY, Tw, Th, Tr); // Calcul de la nouvelle OBB
				while (Collision(&tmpTank, &tile) && tmpY != 0.f)
				{
					// Diminue le dplacement en X
					if (tmpY > 0.f)
						tmpY -= 0.02f;
					else
						tmpY += 0.02f;

					// Si le dpacement est trop faible, on arrete
					if (abs(tmpY) <= 0.03f)
						tmpY = 0.f;

					// Calcul de la nouvelle OBB
					tmpTank = OBB(Tx, Ty + tmpY, Tw, Th, Tr);
				}
			}
		}
	}
	Sprite::Move(tmpX, tmpY);
}

void Tank::Rotate(float rotation, Tank& enemyTank)
{
	// Si on est mort, on bouge pas
	if (m_isExploding) return;

	// Racourci l'appel aux fonctions
	float Tx = this->GetPosition().x;
	float Ty = this->GetPosition().y;
	float Tw = TANK_W;
	float Th = TANK_H;
	float Tr = this->GetRotation();

	// Variables de test
	float tmpRot = rotation;
	OBB tmpTank(Tx, Ty, Tw, Th, Tr + tmpRot);
	OBB tile;
	
	/* Colision avec le dcor */
	for (unsigned int i = static_cast<unsigned int>(((Tx / CELL_SIZE) - ((Tw / CELL_SIZE))) - 1); 
						i < static_cast<unsigned int>(((Tx / CELL_SIZE) + ((Tw / CELL_SIZE))) + 1); 
						++i)
	{
		if (i < 0) continue;
		if (i >= m_map.GetMap().size()) break;
		for (unsigned int j = static_cast<unsigned int>(((Ty / CELL_SIZE) - ((Th / CELL_SIZE))) - 1); 
							j < static_cast<unsigned int>(((Ty / CELL_SIZE) + ((Th / CELL_SIZE))) + 1); 
							++j)
		{
			if (j < 0) continue;
			if (j >= m_map.GetMap().at(i).size()) break;
			else if (m_map.GetMap().at(i).at(j).IsBlocking()) // Le tile n'est pas bloquant
			{
				// Cre l'OBB de la tile  tester actuelle
				tile = OBB(i * CELL_SIZE + CELL_SIZE / 2, j * CELL_SIZE + CELL_SIZE / 2, CELL_SIZE, CELL_SIZE);

				if (Collision(&tmpTank, &tile))
					tmpRot = 0.f;
			}
		}
	}
	Sprite::Rotate(tmpRot);
}

// Tire une bombe
void Tank::ThrowBomb()
{
	// Si on est mort, on tire pas
	if (m_isExploding) return;

	// 2 bombes maximum
	if (m_bombStack.size() < 3 && m_BombClock.GetElapsedTime() / 1000.f >= 1.f)
	{
		m_bombStack.push_back(Bomb(&m_textureBomb, &m_map));
		m_bombStack.at(m_bombStack.size() - 1).SetTextureRect(sf::IntRect(0, 0, BOMB_W, BOMB_MH));
		m_bombStack.at(m_bombStack.size() - 1).SetOrigin(static_cast<float>(BOMB_W), static_cast<float>(BOMB_MH / 2));
		m_bombStack.at(m_bombStack.size() - 1).SetRotation(this->GetRotation());
		m_bombStack.at(m_bombStack.size() - 1).SetPosition(static_cast<float>(cos(PI * this->GetRotation() / 180) * (TANK_W / 2 + BOMB_W)) + this->GetPosition().x,
			static_cast<float>(sin(PI * this->GetRotation() / 180) * (TANK_W / 2 + BOMB_W)) + this->GetPosition().y);
	}
	else if (m_bombStack.size() >= 3)
		m_BombClock.Reset();
}

// Prends des dommages
void Tank::TakeDamages(int dom)
{
	if (m_life - dom >= 0)
		m_life -= dom;
	else
	{
		m_life = 0;
		m_isExploding = true;
	}
}

// Mise  jour de l'animation du tank
void Tank::UpdateAnim()
{
	// Tank en vie >= 75%
	if (!m_isExploding && m_life >= 500)
		m_currentFrameY = 0; // Premire ligne

	// Tank en vie <= 75 %
	else if (!m_isExploding)
		m_currentFrameY = 1; // Deuxime ligne

	// Tank en train d'exploser
	else
		m_currentFrameY = 2; // Troisime ligne

	// Toutes les 0.2 secondes, l'anim du tank bouge
	if (m_animClock.GetElapsedTime() / 1000.f >= 0.06f)
	{
		m_animClock.Reset();

		// Bouge l'image
		if (++m_currentFrameX >= m_nbFrames)
			m_currentFrameX = 0;

		/* Dcommenter pour pouvoir bouger les frames en roulant
		else if (m_isMovingForw)
		{
		if (--m_currentFrameX < 0)
		m_currentFrameX = m_nbFrames - 1;
		m_isMovingForw = false;
		}
		else if (m_isMovingBckw)
		{
		if (++m_currentFrameX >= m_nbFrames)
		m_currentFrameX = 0;
		m_isMovingBckw = false;
		}*/
	}

	// Mets  jour la frame
	SetTextureRect(sf::IntRect(m_currentFrameX * TANK_W, m_currentFrameY * TANK_H, TANK_W, TANK_H));
}

// Mise  jour des bombes
void Tank::UpdateBombs(float timeFactor, Tank& enemyTank)
{
	for (unsigned int i = 0; i < m_bombStack.size(); ++i)
	{
		// Mets  jour la bombe
		m_bombStack.at(i).Update(timeFactor, enemyTank);

		// Si la bombe est morte
		if (!m_bombStack.at(i).m_isAlive)
			m_bombStack.erase(m_bombStack.begin() + i);

		// Si la bombe est en train d'exploser
		else if (m_bombStack.at(i).m_isExploding)
		{
			// Toutes les 0.2 secondes, l'anim de l'explosion change
			if (m_bombStack.at(i).m_animClock.GetElapsedTime() / 1000.f >= 0.06f)
			{
				// Bouge l'image
				if (++m_bombStack.at(i).m_currentFrame >= m_bombStack.at(i).m_nbFrames)
					m_bombStack.erase(m_bombStack.begin() + i);
				else
					m_bombStack.at(i).SetTextureRect(sf::IntRect(m_bombStack.at(i).m_currentFrame * BOMB_W, 0, BOMB_W, BOMB_MH));
			}
		}
	}
}

// Affichage des bombes
void Tank::DrawBombs(sf::RenderTarget &target)
{
	for (unsigned int i = 0; i < m_bombStack.size(); ++i)
		target.Draw(m_bombStack.at(i));
}
