#pragma once
#include <SFML\Graphics.hpp>
#include <string>
#include <vector>
#include "Bomb.h"
#include "OBB.h"
#include "TileMap.h"

const float SPEED = 100.f; // Nb de pixels (dplacement) / seconde
const float SPEED_R = 90.f; // Rotation (nb de degrs) / seconde
const unsigned int TANK_W = 29;
const unsigned int TANK_H = 24;

class Bomb;

class Tank : public sf::Sprite
{
public:
	Tank(const sf::Texture& textureTank, const sf::Texture& textureBomb, const sf::Vector2f& pos, TileMap &map);
	virtual ~Tank(void);

	// Dplacement intuitif
	void MoveForw(float timeFactor, Tank& enemyTank);
	void MoveBckw(float timeFactor, Tank& enemyTank);
	void TurnRight(float timeFactor, Tank& enemyTank);
	void TurnLeft(float timeFactor, Tank& enemyTank);

	// Dplacement prcis
	void Move(float x, float y, Tank& enemyTank);
	void Rotate(float rotation, Tank& enemyTank);

	// Lance une bombe
	void ThrowBomb();

	// Prends des dommages
	void TakeDamages(int dom);

	// Mises  jour
	void UpdateAnim();
	void UpdateBombs(float timeFactor, Tank& enemyTank);

	// Dessine les bombes sur une surface de rendu
	void DrawBombs(sf::RenderTarget &target);

	// Fonctions Get
	int GetLife() const { return m_life; }
	int GetNbFrames() const { return m_nbFrames; }
	float GetMoveSpeed() const { return m_moveSpeed; }
	float GetRotateSpeed() const { return m_rotateSpeed; }
	int GetCurrentFrameX() const { return m_currentFrameX; }
	int GetCurrentFrameY() const { return m_currentFrameY; }
	const sf::Texture& GetTextureTank() const { return m_textureTank; }
	const sf::Texture& GetTextureBomb() const { return m_textureBomb; }
	const std::vector<Bomb>& GetBombStack() const { return m_bombStack; }
	sf::Uint32 GetElapsedTime() const { return m_animClock.GetElapsedTime(); }
	OBB GetOBB() const { return OBB(GetPosition().x, GetPosition().y, TANK_W, TANK_H, GetRotation()); }

	// Fonctions Is
	bool IsMovingForw() const { return m_isMovingForw; }
	bool IsMovingBckw() const { return m_isMovingBckw; }

private:
	int m_life;
	bool m_isExploding;
	float m_moveSpeed;
	float m_rotateSpeed;
	const sf::Texture &m_textureTank;
	const sf::Texture &m_textureBomb;

	// Tableau de jeu pour les colisions
	TileMap &m_map;

	// Animation
	sf::Clock m_animClock;
	int m_nbFrames;
	int m_currentFrameX;
	int m_currentFrameY;
	bool m_isMovingForw; // Est en train d'avancer
	bool m_isMovingBckw; // Est en train de reculer

	// Bombes et mitraille
	sf::Clock m_BombClock;
	std::vector<Bomb> m_bombStack;
};

