#include "TileMap.h"


TileMap::TileMap(const sf::Image &tilemap)
	: m_tilemap(tilemap),
	m_nbRows(SCREEN_WIDTH / CELL_SIZE + 1),
	m_nbColumns(SCREEN_HEIGHT / CELL_SIZE + 1)
{
	// TODO: C'est pas beau ! -> Systme automatique
	// Chargement des textures
	m_textures.push_back(sf::Texture());
	m_textures.back().LoadFromFile("img/ground.jpg");
	m_textures.push_back(sf::Texture());
	m_textures.back().LoadFromFile("img/brickWall.jpg");
	m_textures.push_back(sf::Texture());
	m_textures.back().LoadFromFile("img/tree.jpg");
	m_textures.push_back(sf::Texture());
	m_textures.back().LoadFromFile("img/water.jpg");

	m_renderTexture.Create(SCREEN_WIDTH, SCREEN_HEIGHT);

	UpdateTiles();
	UpdateTexture();
}

TileMap::~TileMap(void)
{
}

void TileMap::SetTileLife(int i, int j, int life)
{
	try {
	m_map.at(i).at(j).SetLife(life);
	}
	catch(std::exception const& e) {
        std::cerr << "ERREUR : " << e.what() << std::endl;
    }
}

void TileMap::AddToTileLife(int i, int j, int life)
{
	try	{
		m_map.at(i).at(j).SetLife(m_map.at(i).at(j).GetLife() + life);
	}
	catch(std::exception const& e) {
		std::cerr << "ERREUR : " << e.what() << std::endl;
	}
}

void TileMap::UpdateTiles()
{
	// Remplissage du tableau
	sf::Vector2f tank1_InitPos(0.f, 0.f), tank2_InitPos(5 * CELL_SIZE, 10 * CELL_SIZE);
	m_map.clear();
	m_map.resize(m_nbRows);
	for (unsigned int i = 0; i < m_map.size(); ++i)
	{
		m_map.at(i).insert(m_map.at(i).begin(), m_nbColumns, Tile(&m_textures.at(0)));
		for (unsigned int j = 0; j < m_map.at(i).size(); ++j)
		{
			// Position des tanks
			if (m_tilemap.GetPixel(i, j) == sf::Color::Green)
				m_tank1_initPos = sf::Vector2f(static_cast<float>(i * CELL_SIZE), static_cast<float>(j * CELL_SIZE));
			else if (m_tilemap.GetPixel(i, j) == sf::Color::Yellow)
				m_tank2_initPos = sf::Vector2f(static_cast<float>(i * CELL_SIZE), static_cast<float>(j * CELL_SIZE));

			// Tilemap
			// TODO: Systme de map plus pouss
			else if (m_tilemap.GetPixel(i, j) == sf::Color::Red) // Mur
			{
				m_map.at(i).at(j).SetAnimTexture(&m_textures.at(1));
				m_map.at(i).at(j).SetBlocking(true);
				m_map.at(i).at(j).SetHoverable(false);
				m_map.at(i).at(j).SetBreakable(false);
			}
			else if (m_tilemap.GetPixel(i, j) == sf::Color::Blue) // Arbres
			{
				m_map.at(i).at(j).SetAnimTexture(&m_textures.at(2));
				m_map.at(i).at(j).SetBlocking(true);
				m_map.at(i).at(j).SetHoverable(false);
				m_map.at(i).at(j).SetBreakable(true);
			}
			else if (m_tilemap.GetPixel(i, j) == sf::Color::Cyan) // Eau
			{
				m_map.at(i).at(j).SetAnimTexture(&m_textures.at(3));
				m_map.at(i).at(j).SetBlocking(true);
				m_map.at(i).at(j).SetHoverable(true);
				m_map.at(i).at(j).SetBreakable(false);
			}
			else												  // Vide
			{
				m_map.at(i).at(j).SetAnimTexture(&m_textures.at(0));
				m_map.at(i).at(j).SetBlocking(false);
				m_map.at(i).at(j).SetHoverable(true);
				m_map.at(i).at(j).SetBreakable(false);
			}
			m_map.at(i).at(j).SetPosition(static_cast<float>(i * CELL_SIZE), static_cast<float>(j * CELL_SIZE));
		}
	}
	UpdateTexture();
}

void TileMap::UpdateTexture()
{
	m_renderTexture.Clear(sf::Color(128, 128, 128));
	for (unsigned int i = 0; i < m_map.size(); ++i)
		for (unsigned int j = 0; j < m_map.at(i).size(); ++j)
			m_renderTexture.Draw(m_map.at(i).at(j));
	m_renderTexture.Display();
}

void TileMap::UpdateAnims()
{
	for (unsigned int i = 0; i < m_map.size(); ++i)
		for (unsigned int j = 0; j < m_map.at(i).size(); ++j)
			m_map.at(i).at(j).UpdateAnim();
}


void TileMap::Draw(sf::RenderTarget& target)
{
	UpdateAnims();
	UpdateTexture();
	sf::Sprite sprite(m_renderTexture.GetTexture());
	target.Draw(sprite);
}